@@ -4,28 +4,15 @@
 
 import sys
 import argparse
-import subprocess
 import os
-import requests
 import json
 import shutil
 import platform
-import stat
 import uuid
 import traceback
+from urllib.parse import urlencode
 from . import http
-
-verbose = False
-
-def get_python_link_name(pydll_path, os_name):
-    if os_name == "linux":
-        for so in os.listdir(pydll_path):
-            if so.startswith("libpython") and not so.endswith(".so") and so.find(".so") != -1:
-                basename = os.path.basename(so[3:so.find(".so")])
-                full_path = os.path.join(pydll_path, so)
-                return basename, full_path
-    return None, None
-
+from . import utils
 
 class ChangeCWD:
     def __init__(self, dir):
@@ -34,13 +21,11 @@ class ChangeCWD:
     
     def __enter__(self):
         os.chdir(self.dir)
-        if verbose:
-            print(f"Enter {self.dir}")
+        utils.vprint(f"Enter {self.dir}")
 
     def __exit__(self, *args, **kwargs):
         os.chdir(self.old)
-        if verbose:
-            print(f"Leave {self.dir}, Enter {self.old}")
+        utils.vprint(f"Leave {self.dir}, Enter {self.old}")
 
 
 class Cmd:
@@ -55,13 +40,30 @@ class Cmd:
     def help(self):
         self.parser.print_help()
 
-    def run(self, args):
+    def hello(self):
+        print(
+        "      ___                       ___                 \n"+
+        "     /\\__\\          ___        /\\__\\          ___   \n"+
+        "    /:/  /         /\\  \\      /:/ _/_        /\\  \\  \n"+
+        "   /:/__/          \\:\\  \\    /:/ /\\__\\       \\:\\  \\ \n"+
+        "  /::\\__\\____      /::\\__\\  /:/ /:/ _/_      /::\\__\\\n"+
+        " /:/\\:::::\\__\\  __/:/\\/__/ /:/_/:/ /\\__\\  __/:/\\/__/       Welcome to KIWI World ! Enjoy One-Click Magic!\n"+
+        " \\/_|:|~~|~    /\\/:/  /    \\:\\/:/ /:/  / /\\/:/  /  \n"+
+        "    |:|  |     \\::/__/      \\::/_/:/  /  \\::/__/                     https://www.zifuture.com\n"+
+        "    |:|  |      \\:\\__\\       \\:\\/:/  /    \\:\\__\\    \n"+
+        "    |:|  |       \\/__/        \\::/  /      \\/__/    \n"+
+        "     \\|__|                     \\/__/                \n"
+        "\n"
+        "You can use 'kiwi --help' to show the more message."
+        )
+
+    def run(self, args, addi_args):
         args = self.parser.parse_args(args)
         if args.cmd is None:
-            self.help()
+            self.hello()
             return False
 
-        return self.actions.private_run_cmd(**args.__dict__)
+        return self.actions.private_run_cmd(args, addi_args)
 
 
 class Config:
@@ -74,7 +76,7 @@ class Config:
         self.KIWI_ROOT   = os.path.realpath(os.path.dirname(os.path.abspath(__file__)))
         self.CWD         = os.path.abspath(os.path.curdir)
         self.PYLIB_DIR   = os.path.join(sys.exec_prefix, "lib")
-        self.PYLIB_NAME, self.PYLIB_PATH = get_python_link_name(self.PYLIB_DIR, self.OS_NAME)
+        self.PYLIB_NAME, self.PYLIB_PATH = utils.get_python_link_name(self.PYLIB_DIR, self.OS_NAME)
         self.DATA_DIR    = os.path.join(self.CACHE_ROOT, "data")
         self.PKG_DIR     = os.path.join(self.CACHE_ROOT, "pkg")
         self.LIB_DIR     = os.path.join(self.CACHE_ROOT, "lib")
@@ -151,28 +153,27 @@ class Actions:
         self.app = app
         self.cfg : Config = app.cfg
 
-    def private_run_cmd(self, cmd, **kwargs):
+    def private_run_cmd(self, args, _addi_args):
+        
+        cmd = args.cmd
         if not hasattr(self, cmd):
             return False
 
-        return getattr(self, cmd)(argparse.Namespace(**kwargs))
+        del args.__dict__["cmd"]
+        return getattr(self, cmd)(args, _addi_args)
 
     def __rmtree(self, dir):
         dir = dir.strip()
         if dir == "." or dir == ".." or dir == "/":
-            if verbose:
-                print(f"Can not remove directory [{dir}]")
+            utils.vprint(f"Can not remove directory [{dir}]")
             return
 
         if os.path.exists(dir):
-            if verbose:
-                print(f"Remove directory: {dir}")
-
+            utils.vprint(f"Remove directory: {dir}")
             shutil.rmtree(dir)
 
     def __run_py(self, file, args):
-        if verbose:
-            print(f"Run python script file: {file} at {os.getcwd()}")
+        utils.vprint(f"Run python script file: {file} at {os.getcwd()}")
 
         try:
             code_dir = os.path.realpath(os.path.dirname(file))
@@ -183,14 +184,16 @@ class Actions:
             sys.path.insert(0, code_dir)
             m = __import__(code_name, globals(), locals(), ["*"])
             os.remove(temp_code_file)
-            return getattr(m, "run")(self.app, args)
+            result =  getattr(m, "run")(self.app, args)
         except Exception as e:
             traceback.print_exc()
-            return False
+            result = False
+        
+        self.__rmtree(".kiwi/__pycache__")
+        return result
 
     def __run_bash(self, file):
-        if verbose:
-            print(f"Run bash script file: {file} at {os.getcwd()}")
+        utils.vprint(f"Run bash script file: {file} at {os.getcwd()}")
 
         try:
             code_dir = os.path.realpath(os.path.dirname(file))
@@ -210,8 +213,7 @@ class Actions:
         try:
             dir = os.path.realpath(os.path.dirname(file))
             with ChangeCWD(dir):
-                if verbose:
-                    print(f"Run link script file: {file} at {os.getcwd()}")
+                utils.vprint(f"Run link script file: {file} at {os.getcwd()}")
 
                 dic = self.cfg.get_dict()
                 for name in dic:
@@ -229,8 +231,7 @@ class Actions:
                         if len(ps) == 2:
                             fa = os.path.abspath(ps[0])
                             fb = os.path.abspath(ps[1])
-                            if verbose:
-                                print(f"Run link command: ln -s \"{fa}\" \"{fb}\"")
+                            utils.vprint(f"Run link command: ln -s \"{fa}\" \"{fb}\"")
 
                             if os.system(f"ln -s \"{fa}\" \"{fb}\"") != 0:
                                 print(f"Failed to run command: ln -s \"{fa}\" \"{fb}\"")
@@ -243,8 +244,7 @@ class Actions:
                                 return
 
                             fb = os.path.abspath(ps[0][:p+3])
-                            if verbose:
-                                print(f"Run link command: ln -s \"{fa}\" \"{fb}\"")
+                            utils.vprint(f"Run link command: ln -s \"{fa}\" \"{fb}\"")
 
                             if os.system(f"ln -s \"{fa}\" \"{fb}\"") != 0:
                                 print(f"Failed to run command: ln -s \"{fa}\" \"{fb}\"")
@@ -254,22 +254,19 @@ class Actions:
                             return False
 
                     elif op == "cmd":
-                        if verbose:
-                            print(f"Run command: {param}")
+                        utils.vprint(f"Run command: {param}")
 
                         if os.system(param) != 0:
                             print(f"Invalid command: {param}")
                             return False
-            if verbose:
-                print(f"Remove {file}")
+            utils.vprint(f"Remove {file}")
             os.remove(file)
             return True
         except Exception as e:
             traceback.print_exc()
             return False
 
-
-    def config(self, args : argparse.Namespace):
+    def config(self, args : argparse.Namespace, addi_args):
 
         if args.key is None and args.value is None:
             print(self.cfg)
@@ -288,48 +285,65 @@ class Actions:
         print("Success!")
         return True
 
-    def auth(self, args : argparse.Namespace):
+    def auth(self, args : argparse.Namespace, addi_args):
         setattr(self.cfg, "ACCESS_TOKEN", args.token)
         self.cfg.save()
         print("Success!")
         return True
 
-    def get(self, args : argparse.Namespace):
+    def get(self, args : argparse.Namespace, addi_args):
         repo = args.repo
-        owner, proj = repo.split("/")
+        if repo.find("/") == -1:
+            print(f"Invalid repo name: {repo}")
+            return False
 
+        owner, proj = repo.split("/")
         file = os.path.join(self.cfg.CODE_DIR, repo + ".zip")
         fileurl = f"{self.cfg.SERVER}/public/repo/zip/{repo}?accessToken={self.cfg.ACCESS_TOKEN}"
         md5url  = f"{self.cfg.SERVER}/public/repo/shortinfo/{repo}?accessToken={self.cfg.ACCESS_TOKEN}"
-        if not http.require_file_and_check_md5(fileurl, md5url, file, f"Getting {repo}", args.update, verbose)[0]:
+        if not http.require_file_and_check_md5(fileurl, md5url, file, f"Getting {repo}", args.update, utils.verbose)[0]:
             print(f"Failed to get repo {repo}.")
             return False
 
         if args.save is not None:
             proj = args.save
 
-        self.__rmtree(proj)
-        http.extract_zip_to(file, proj, verbose)
+        if args.rmtree:
+            self.__rmtree(proj)
+
+        http.extract_zip_to(file, proj, utils.verbose)
 
         if not args.disable_run:
             with ChangeCWD(proj):
                 auto_config_file = ".kiwi/auto.py"
                 if os.path.exists(auto_config_file):
+                    utils.vprint(f"Run automatic script {auto_config_file}")
                     self.__run_py(auto_config_file, args)
+                else:
+                    utils.vprint(f"Not exist {auto_config_file}")
 
                 auto_config_file = ".kiwi/auto.sh"
                 if os.path.exists(auto_config_file):
+                    utils.vprint(f"Run automatic script {auto_config_file}")
                     self.__run_bash(auto_config_file)
+                else:
+                    utils.vprint(f"Not exist {auto_config_file}")
+        else:
+            utils.vprint(f"Do not run any automatic scripts, because disable_run is set.")
         return True
 
-    def getd(self, args : argparse.Namespace):
+    def getd(self, args : argparse.Namespace, addi_args):
         data = args.data
+        if data.find("/") == -1:
+            print(f"Invalid data name: {data}")
+            return False
+
         owner, dname = data.split("/")
 
         file = os.path.join(self.cfg.DATA_DIR, data + ".zip")
         fileurl = f"{self.cfg.SERVER}/public/data/download/{data}?accessToken={self.cfg.ACCESS_TOKEN}"
         md5url  = f"{self.cfg.SERVER}/public/data/shortinfo/{data}?accessToken={self.cfg.ACCESS_TOKEN}"
-        ok, info = http.require_file_and_check_md5(fileurl, md5url, file, f"Getting {data}", args.update, verbose)
+        ok, info = http.require_file_and_check_md5(fileurl, md5url, file, f"Getting {data}", args.update, utils.verbose)
         if not ok:
             print(f"Failed to install data {data}.")
             return False
@@ -349,19 +363,25 @@ class Actions:
         else:
             if args.save is not None:
                 dname = args.save
-
-            self.__rmtree(dname)
-            http.extract_zip_to(file, dname, verbose)
+            
+            if args.rmtree:
+                self.__rmtree(dname)
+            http.extract_zip_to(file, dname, utils.verbose)
             return True
 
-    def install(self, args : argparse.Namespace):
+    def install(self, args : argparse.Namespace, addi_args):
+
         pkg = args.pkg
+        if pkg.find("/") == -1:
+            print(f"Invalid pkg name: {pkg}")
+            return False
+
         owner, dname = pkg.split("/")
 
         file = os.path.join(self.cfg.PKG_DIR, pkg + ".zip")
         fileurl = f"{self.cfg.SERVER}/public/pkg/download/{pkg}?accessToken={self.cfg.ACCESS_TOKEN}"
         md5url  = f"{self.cfg.SERVER}/public/pkg/shortinfo/{pkg}?accessToken={self.cfg.ACCESS_TOKEN}"
-        ok, info = http.require_file_and_check_md5(fileurl, md5url, file, f"Getting {pkg}", args.update, verbose)
+        ok, info = http.require_file_and_check_md5(fileurl, md5url, file, f"Getting {pkg}", args.update, utils.verbose)
         if not ok:
             print(f"Failed to install pkg {pkg}.")
             return False
@@ -384,23 +404,29 @@ class Actions:
             else:
                 install_to = args.save
 
-            self.__rmtree(install_to)
-            http.extract_zip_to(file, install_to, verbose)
+            if args.rmtree:
+                self.__rmtree(install_to)
+
+            http.extract_zip_to(file, install_to, utils.verbose)
 
             print(f"Install to {install_to}")
-            return self.__run_link(os.path.abspath(os.path.join(install_to, ".link")))
+            link_file = os.path.abspath(os.path.join(install_to, ".link"))
+            if os.path.exists(link_file):
+                return self.__run_link(link_file)
+            return True
 
-    def clean(self, args : argparse.Namespace):
+    def clean(self, args : argparse.Namespace, addi_args):
         self.__rmtree(self.cfg.CODE_DIR)
         self.__rmtree(self.cfg.PKG_DIR)
         self.__rmtree(self.cfg.DATA_DIR)
         os.makedirs(self.cfg.DATA_DIR, exist_ok=True)
         os.makedirs(self.cfg.PKG_DIR,  exist_ok=True)
         os.makedirs(self.cfg.CODE_DIR, exist_ok=True)
+        return True
 
-    def run(self, args : argparse.Namespace):
+    def run(self, args : argparse.Namespace, addi_args):
 
-        file    = args.file
+        file    = args.runfile
         workdir = args.workdir
 
         if file is None:
@@ -434,8 +460,8 @@ class Actions:
 
         with ChangeCWD(workdir):
 
-            if args.args is not None:
-                allargs = args.args.split(" ")
+            if addi_args is not None:
+                allargs = addi_args
                 kwargs_map = {}
                 args_list  = []
                 for item in allargs:
@@ -458,6 +484,28 @@ class Actions:
                 return self.__run_bash(file)
             return False
 
+    def search(self, args : argparse.Namespace, addi_args):
+        param = urlencode({"key": args.key, "accessToken": self.cfg.ACCESS_TOKEN})
+        url = f"{self.cfg.SERVER}/public/search/list?" + param
+        data = http.request_text(url)
+        if data is None:
+            return False
+
+        print(f"Search {len(data)} results using {args.key}")
+        for i, (size, shortDescription, createTime, dtype, path) in enumerate(data):
+            
+            i += 1
+            if dtype in ["Data", "Package"]:
+                if shortDescription is not None:
+                    print(f"{i}[{dtype}]: {path}, {utils.format_size(size)}, {shortDescription}")
+                else:
+                    print(f"{i}[{dtype}]: {path}, {utils.format_size(size)}")
+            else:
+                if shortDescription is not None:
+                    print(f"{i}[{dtype}]: {path}, {shortDescription}")
+                else:
+                    print(f"{i}[{dtype}]: {path}")
+        return True
 
 class Application:
     def __init__(self):
@@ -470,7 +518,6 @@ class Application:
 
     def run_with_command(self, args=None)->bool:
 
-        global verbose
         if args is not None and isinstance(args, str):
             args = args.split(" ")
         elif args is None:
@@ -479,8 +526,30 @@ class Application:
         for i in range(len(args)):
             if args[i] == "-verbose":
                 del args[i]
-                verbose = True
+                utils.verbose = True
                 break
+        
+        addi_args = None
+        if len(args) > 0:
+            if(args[0] == "run"):
+                addi_args = []
+                old_args  = []
+                skip = [False] * len(args) 
+                for i in range(1, len(args)):
+                    if skip[i]: continue
+
+                    arg = args[i]
+                    if not (arg.startswith("-runfile") or arg.startswith("-workdir")):
+                        addi_args.append(arg)
+                    else:
+                        if arg.find("=") == -1:
+                            if i + 1 < len(args):
+                                old_args.append(arg)
+                                old_args.append(args[i + 1])
+                                skip[i + 1] = True
+                        else:
+                            old_args.append(arg)
+                args = ["run"] + old_args
 
         cmd = Cmd(self.actions)
         c = cmd.add_cmd("config", "Configure")
@@ -491,11 +560,11 @@ class Application:
         c.add_argument("repo", type=str, help="repo name")
         c.add_argument("-update", action="store_true", help="Force update")
         c.add_argument("-save", type=str, help="Unzipped folder")
+        c.add_argument("-rmtree", action="store_true", help="Unzip the data after remove the save folder.")
         c.add_argument("-disable-run", action="store_true", help="Disable auto run")
 
-        c = cmd.add_cmd("run", "Run script")
-        c.add_argument("-args", type=str, help="script augments")
-        c.add_argument("-file", type=str, help="script file, default is .kiwi/auto.py or .kiwi/auto.sh")
+        c = cmd.add_cmd("run", "Run the project automation script")
+        c.add_argument("-runfile", type=str, help="script file, default is .kiwi/auto.py or .kiwi/auto.sh")
         c.add_argument("-workdir", type=str, help="workspace dir, default is solution dir")
 
         c = cmd.add_cmd("auth", "Set auth ACCESS_TOKEN")
@@ -505,6 +574,7 @@ class Application:
         c.add_argument("data", type=str, help="data name")
         c.add_argument("-update", action="store_true", help="Force update")
         c.add_argument("-save", type=str, help="Unzipped folder")
+        c.add_argument("-rmtree", action="store_true", help="Unzip the data after remove the save folder.")
         c.add_argument("-disable-run", action="store_true", help="Disable auto run")
 
         c = cmd.add_cmd("clean", "Clean cache file")
@@ -513,5 +583,9 @@ class Application:
         c.add_argument("pkg", type=str, help="pkg name")
         c.add_argument("-update", action="store_true", help="Force update")
         c.add_argument("-save", type=str, help="Unzipped folder")
+        c.add_argument("-rmtree", action="store_true", help="Unzip the data after remove the save folder.")
         c.add_argument("-disable-run", action="store_true", help="Disable auto run")
-        return cmd.run(args)
\ No newline at end of file
+
+        c = cmd.add_cmd("search", "Search solution / data / package")
+        c.add_argument("key", type=str, help="search name")
+        return cmd.run(args, addi_args)
\ No newline at end of file
