@@ -1,6 +1,8 @@
-
 import akiwi
 
-if __name__ == "__main__":
+def main():
     app = akiwi.Application()
-    app.run_with_command()
\ No newline at end of file
+    return app.run_with_command()
+
+if __name__ == "__main__":
+    main()
\ No newline at end of file
